import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http_mock_adapter/http_mock_adapter.dart';
import 'package:rokeet/src/network/client/dio/dio_http_client.dart';
import 'package:rokeet/src/network/network.dart';

void main() {
  group("Dio HttpClient", () {
    late HttpClient _httpClient;
    late DioAdapter _dioAdapter;
    setUp(() {
      _dioAdapter = DioAdapter();
      _httpClient = DioHttpClient.builder()
          .withBaseUrl("http://localhost")
          .withClientAdapter(_dioAdapter)
          .build();
    });

    test("GET should return a HTTP STATUS 200", () async {
      _dioAdapter.onGet("/", (request) => request.reply(200, ""));
      Response<dynamic> response = await _httpClient.get("/");
      expect(response.statusCode, HttpStatus.ok);
    });

    test("GET should return a HTTP STATUS 400", () async {
      _dioAdapter.onGet("/", (request) => request.reply(400, ""));
      Response<dynamic> response = await _httpClient.get("/");
      expect(response.statusCode, HttpStatus.badRequest);
    });

    test("POST should return a HTTP STATUS 200", () async {
      _dioAdapter.onPost("/", (request) => request.reply(200, ""));
      Response<dynamic> response = await _httpClient.post("/");
      expect(response.statusCode, HttpStatus.ok);
    });
    test("POST should return a HTTP STATUS 400", () async {
      _dioAdapter.onPost("/", (request) => request.reply(400, ""));
      Response<dynamic> response = await _httpClient.post("/");
      expect(response.statusCode, HttpStatus.badRequest);
    });

    test("PUT should return a HTTP STATUS 200", () async {
      _dioAdapter.onPut("/", (request) => request.reply(200, ""));
      Response<dynamic> response = await _httpClient.put("/");
      expect(response.statusCode, HttpStatus.ok);
    });
    test("PUT should return a HTTP STATUS 400", () async {
      _dioAdapter.onPut("/", (request) => request.reply(400, ""));
      Response<dynamic> response = await _httpClient.put("/");
      expect(response.statusCode, HttpStatus.badRequest);
    });

    test("PATCH should return a HTTP STATUS 200", () async {
      _dioAdapter.onPatch("/", (request) => request.reply(200, ""));
      Response<dynamic> response = await _httpClient.patch("/");
      expect(response.statusCode, HttpStatus.ok);
    });
    test("PATCH should return a HTTP STATUS 400", () async {
      _dioAdapter.onPatch("/", (request) => request.reply(400, ""));
      Response<dynamic> response = await _httpClient.patch("/");
      expect(response.statusCode, HttpStatus.badRequest);
    });

    test("DELETE should return a HTTP STATUS 200", () async {
      _dioAdapter.onDelete("/", (request) => request.reply(200, ""));
      Response<dynamic> response = await _httpClient.delete("/");
      expect(response.statusCode, HttpStatus.ok);
    });
    test("DELETE should return a HTTP STATUS 400", () async {
      _dioAdapter.onDelete("/", (request) => request.reply(400, ""));
      Response<dynamic> response = await _httpClient.delete("/");
      expect(response.statusCode, HttpStatus.badRequest);
    });
  });
}
